package io.kiteboard.simplepiano;

import android.content.Context;
import android.content.res.AssetManager;
import android.media.AudioAttributes;
import android.media.AudioManager;
import android.media.SoundPool;
import android.os.Bundle;
import android.os.PowerManager;
import android.os.PowerManager.WakeLock;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.View;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.TextView;

import java.io.DataInputStream;
import java.io.IOException;
import java.net.Socket;
import java.net.UnknownHostException;

public class MainActivity extends AppCompatActivity {

    static final String TAG = "Piano";

    Socket client = null;
    boolean isUiVisible = false;
    PowerManager powerManager;
    WakeLock wakeLock;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        // Connect to the piano server (running in the chroot environment)
        connectToPiano();

        // Setup Audio
        setupSoundPool();

        // Ensure the device does not sleep when the app is running
        // Note that "app" != "activity UI" in Android)
        powerManager = (PowerManager) getSystemService(POWER_SERVICE);
        wakeLock = powerManager.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK,
                "PianoWaklock");
        wakeLock.acquire();
    }

    SoundPool mSoundPool;
    int[] noteSound= new int[13];

    String [] soundFileName = {
      //      "39148__jobro__piano-ff-001.wav",
      //      "39149__jobro__piano-ff-002.wav",
      //      "39150__jobro__piano-ff-003.wav",
            "39151__jobro__piano-ff-004.wav",
            "39152__jobro__piano-ff-005.wav",
            "39153__jobro__piano-ff-006.wav",
            "39154__jobro__piano-ff-007.wav",
            "39155__jobro__piano-ff-008.wav",
            "39156__jobro__piano-ff-009.wav",
            "39157__jobro__piano-ff-010.wav",
            "39158__jobro__piano-ff-011.wav",
            "39159__jobro__piano-ff-012.wav",
            "39160__jobro__piano-ff-013.wav",
            "39161__jobro__piano-ff-014.wav",
            "39162__jobro__piano-ff-015.wav",
            "39163__jobro__piano-ff-016.wav",
            "39164__jobro__piano-ff-017.wav",
            "39165__jobro__piano-ff-018.wav",
            "39166__jobro__piano-ff-019.wav",
            "39167__jobro__piano-ff-020.wav",
            "39168__jobro__piano-ff-021.wav",
            "39169__jobro__piano-ff-022.wav",
            "39170__jobro__piano-ff-023.wav",
            "39171__jobro__piano-ff-024.wav",
            "39172__jobro__piano-ff-025.wav",
            "39173__jobro__piano-ff-026.wav",
            "39174__jobro__piano-ff-027.wav",
            "39175__jobro__piano-ff-028.wav",
            "39176__jobro__piano-ff-029.wav",
            "39177__jobro__piano-ff-030.wav",
            "39178__jobro__piano-ff-031.wav",
            "39179__jobro__piano-ff-032.wav",
            "39180__jobro__piano-ff-033.wav",
            "39181__jobro__piano-ff-034.wav",
            "39182__jobro__piano-ff-035.wav",
            "39183__jobro__piano-ff-036.wav",
            "39184__jobro__piano-ff-037.wav",
            "39185__jobro__piano-ff-038.wav",
            "39186__jobro__piano-ff-039.wav",
            "39187__jobro__piano-ff-040.wav",
            "39188__jobro__piano-ff-041.wav",
            "39189__jobro__piano-ff-042.wav",
            "39190__jobro__piano-ff-043.wav",
            "39191__jobro__piano-ff-044.wav",
            "39193__jobro__piano-ff-045.wav",
            "39194__jobro__piano-ff-046.wav",
            "39195__jobro__piano-ff-047.wav",
            "39196__jobro__piano-ff-048.wav",
            "39197__jobro__piano-ff-049.wav",
            "39198__jobro__piano-ff-050.wav",
            "39199__jobro__piano-ff-051.wav",
            "39200__jobro__piano-ff-052.wav",
            "39201__jobro__piano-ff-053.wav",
            "39202__jobro__piano-ff-054.wav",
            "39203__jobro__piano-ff-055.wav",
            "39204__jobro__piano-ff-056.wav",
            "39205__jobro__piano-ff-057.wav",
            "39206__jobro__piano-ff-058.wav",
            "39207__jobro__piano-ff-059.wav",
            "39208__jobro__piano-ff-060.wav",
            "39209__jobro__piano-ff-061.wav",
            "39210__jobro__piano-ff-062.wav",
            "39211__jobro__piano-ff-063.wav",
            "39212__jobro__piano-ff-064.wav",
            "39213__Piano.ff.Db6.wav",
            "39214__Piano.ff.D6.wav",
            "39215__Piano.ff.Eb6.wav",
            "39216__Piano.ff.E6.wav",
            "39217__Piano.ff.F6.wav",
            "39218__Piano.ff.Gb6.wav",
            "39219__Piano.ff.G6.wav",
            "39220__Piano.ff.Ab6.wav",
            "39221__Piano.ff.A6.wav",
            "39222__Piano.ff.Bb6.wav",
            "39223__Piano.ff.B6.wav",
            "39224__Piano.ff.C7.wav",
            "39225__Piano.ff.Db7.wav",
            "39226__Piano.ff.D7.wav",
            "39227__Piano.ff.Eb7.wav",
            "39228__Piano.ff.E7.wav",
            "39229__Piano.ff.F7.wav",
            "39230__Piano.ff.Gb7.wav",
            "39231__Piano.ff.G7.wav",
            "39232__Piano.ff.Ab7.wav",
            "39233__jobro__piano-ff-085.wav",
            "39234__jobro__piano-ff-086.wav",
            "39235__jobro__piano-ff-087.wav",
            "39236__jobro__piano-ff-088.wav",
    };
    void setupSoundPool() {
        AssetManager am = this.getAssets();
        Context myContext = getApplicationContext();

        AudioAttributes attrib = new AudioAttributes.Builder().
                setUsage(AudioAttributes.USAGE_GAME).
                setContentType(AudioAttributes.CONTENT_TYPE_SONIFICATION).
                build();
        mSoundPool = new SoundPool.Builder().setAudioAttributes(attrib).setMaxStreams(1).build();
        for(int i=0;i<13;i++) {
            //noteSound = mSoundPool.load(myContext, R.raw.o1_db6_v, 1);
            String filename;
            int octave = 3;

            filename = "sounds/piano/"+soundFileName[octave*12+i];
            try {
                noteSound[i] = mSoundPool.load(am.openFd(filename), i);
            } catch (IOException e) {
                Log.e(TAG, "Unable to get " + filename);
            }
        }
        //noteSound = mSoundPool.load(myContext, R.raw.game_over, 1);
    }

    void releaseSoundPool() {
        mSoundPool.release();
        mSoundPool = null;
    }

    @Override
    protected void onResume() {
        super.onResume();
        isUiVisible = true;
    }

    @Override
    protected void onPause() {
        isUiVisible = false;
        super.onPause();
    }

    protected void onDestroy() {
        super.onDestroy();

        try {
            client.close();
        } catch (IOException e) {

        }
        client = null;
        releaseSoundPool();

        wakeLock.release();
    }

    private void connectToPiano() {
        Thread thread = new Thread(new Runnable() {
            @Override
            public void run() {
                // Simple thread that connects to the Piano server
                // gets input line by line & processes it
                try {
                    try {
                        client = new Socket("localhost", 1501);
                    } catch(UnknownHostException e) {
                    } catch(IOException e) {
                    }

                    Log.i(TAG, "Connected to server");
                    DataInputStream inStream = new DataInputStream(client.getInputStream());
                    while(true) {
                        String s = inStream.readLine();
                        Log.e(TAG, s);
                        uiAction(s);
                    }
                } catch (IOException e ){
                }
            }
        });
        thread.start();
    }

    private void uiAction(final String s) {
        // Take action on events coming from piano server
        // depending on whether the "activity" is visible
        String [] parts = s.split(" ");
        final int noteIdx = Integer.parseInt(parts[1]);
        if(parts[2].equals("True")) { // Key pressed ?

            // Update the UI if it is visible
            if (isUiVisible) {
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        TextView tv = (TextView) findViewById(R.id.tvText);
                        tv.setText(s);
                        }
                });
            }

            // play the relevant note
            playNote(noteIdx);
        }
    }

    public void playNote(int noteIdx) {
        // range check for note number to prevent crash
        if((noteIdx>=0) && (noteIdx<noteSound.length)) {
            mSoundPool.play(noteSound[noteIdx], 1.0f, 1.0f, 0, 0, 1.0f);
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }
}
